""" basic commands. """

__version__ = 1

## imports

import logging
import os
import readline
import threading
import time
import ob.tables

from . import Object, edit, format, get_type, k, workdir, __version__
from .db import days, elapsed, last_fn
from .loader import table
from .term import set_completer

## defines

def __dir__():
    return ("ed", "find", "meet", "rm", "show", "undel")

## commands

def ed(event):
    if len(event.args) == 0:
        fns = sorted({x.split(".")[-1].lower() for x in os.listdir(os.path.join(ob.workdir, "store"))})
        event.reply("|".join(fns))
        return
    if len(event.args) == 1 and not event.selector:
        event.options += ",f"
        find(event)
        return
    nr = 0
    objs = k.db.find(event.match, event.selector, event.index, event.delta)
    for o in objs:
        got = edit(o, event.setter)
        if got:
            o.save()
            nr += 1
        event.reply("edit %s" % nr)

def find(event):
    if "k" in event.options:
        o = k.db.last(event.match)
        if o:
            event.reply("|".join(sorted({x for x in o.keys() if not x.startswith("_")})))
            return
    if len(event.args) == 0:
        fns = sorted({x.split(".")[-1].lower() for x in os.listdir(os.path.join(ob.workdir, "store"))})
        if fns:
            event.reply("|".join(fns))
        return
    elif len(event.args) == 1 and not event.selector:
        fn = last_fn(event.match)
        if fn:
            o = Object().load(fn)
            if o:
                res = sorted({x.split(".")[-1].lower() for x in o.keys()})
                if len(res) > 1:
                    event.reply("|".join(res))
                else:
                    for a in res:
                        if a not in event.selector:
                            event.selector[a] = None
                        if a not in event.dkeys:
                            event.dkeys.append(a)
    nr = -1
    for o in k.db.find(event.match, event.selector, event.index, event.delta):
        nr += 1
        txt = ""
        if "k" in event.options:
            event.reply("|".join(o.keys()))
            return
        if "d" in event.options:
            event.reply(str(o))
            continue
        full = False
        if "f" in event.options:
            full = True
        if event.dkeys:
            txt = "%s %s" % (event.index or nr, format(o, event.dkeys, full))
        else:
            txt = "%s %s" % (event.index or nr, format(o, full=full))
        if "t" in event.options:
            txt += " " + days(o._path)
        txt = txt.rstrip()
        if txt:
            event.reply(txt)

## commands

def meet(event):
    if not event.args:
        event.reply("meet origin [permissions]")
        return
    try:
        origin, *perms = event.args[:]
    except ValueError:
        event.reply("|".join(sorted(k.users.userhosts.keys())))
        return
    origin = k.users.userhosts.get(origin, origin)
    u = k.users.meet(origin, perms)
    event.reply("added %s" % u.user)

def rm(event):
    if not event.args:
        event.reply("rm <selector>")
        return
    import time
    st = time.time()
    nr = -1
    for o in k.db.find(event.match, event.selector, event.index, event.delta):
        if not o:
            continue
        nr += 1
        o._deleted = True
        o.save()

def show(event):
    if not event.args:
        event.reply("config|cmds|fleet|kernel|tasks|version")
        return
    bot = k.fleet.get_bot(event.orig)
    cmd = event.args[0]
    if cmd == "config":
        if len(event.args) == 2:
            cfg = k.db.last("%s.Cfg" % event.args[1].lower())
            event.reply(cfg)
        else:
            event.reply(k.cfg)
    elif cmd == "cmds":
        event.reply("|".join(sorted(bot.handlers.keys())))
    elif cmd == "fleet":
        try:
            index = int(event.args[1])
            event.reply(k.fleet.bots[index])
            return
        except (ValueError, IndexError):
            event.reply([get_type(x) for x in k.fleet.bots])
    elif cmd == "kernel":
        event.reply(k)
    elif cmd == "tasks":
        psformat = "%-8s %-60s"
        result = []
        for thr in sorted(threading.enumerate(), key=lambda x: x.getName()):
            if str(thr).startswith("<_"):
                continue
            d = vars(thr)
            o = Object()
            o.update(d)
            if getattr(o, "sleep", None):
                up = o.sleep - int(time.time() - o.state.latest)
            else:
                up = int(time.time() - k.state.starttime)
            result.append((up, thr.getName(), o))
        nr = -1
        for up, thrname, o in sorted(result, key=lambda x: x[0]):
            nr += 1
            res = "%s %s" % (nr, psformat % (elapsed(up), thrname[:60]))
            if res.strip():
                event.reply(res)
    elif cmd == "uptime":
        event.reply(elapsed(time.time() - k.state.starttime))
    elif cmd == "version":
        bot = k.fleet.get_bot(event.orig)
        for name, mod in table.items():
            if event.args and event.args[0] == name:
                ver = getattr(mod, "__version__", None)
                if ver:
                    event.reply("%s %s" % (name, ver))
        event.reply("%s %s" % (k.cfg.name.upper(), k.cfg.version))

def undel(event):
    if not event.args:
        event.reply("undel <selector>")
        return
    st = time.time()
    nr = -1
    for o in k.db.all(event.match, event.selector, event.index, event.delta):
        nr += 1
        o._deleted = False
        o.save()
    event.reply("ok %s %s" % (nr+1, elapsed(time.time()-st)))
