""" command parsing. """

__version__ = 1

def __dir__():
    return ("Command", "Token")

## imports

import ob

## classes

class Token(ob.Object):

    def __init__(self):
        super().__init__()
        self.arg = ""
        self.chk = ""
        self.dkey = ""
        self.down = None
        self.equal = ""
        self.ignore = ""
        self.index = None
        self.match = ""
        self.option = ""
        self.selector = ""
        self.setter = ""
        self.up = None
        self.value = ""

    def parse(self, nr, word):
        if nr == 0:
            if word.startswith("!"):
                word = word[1:]
            if word:
                self.chk = word
            return
        if word.startswith("-"):
            try:
                self.down = int(word[1:])
            except ValueError:
                pass
            self.option += word[1:]
            return
        try:
            self.index = int(word)
            self.arg = word
            return
        except ValueError:
            pass
        if nr == 1:
            self.match = ob.tables.names.get(word, word)
            self.arg = word
            return        
        if "http" in word:
            self.value = word
            self.arg = word
            return
        if word.startswith("+"):
            try:
                self.up = int(word[1:])
                return
            except ValueError:
                pass
        if word.endswith("-"):
            self.ignore = word[:-1]
            word = self.ignore
        if "==" in word:
            self.selector, self.value = word.split("==")
            self.dkey = self.selector
        elif "=" in word:
            self.setter, self.value = word.split("=")
            self.dkey = self.setter
        else:
            self.arg = word
            self.dkey = word
            self.value = word
        if nr == 2 and not self.selector and not self.setter:
            self.selector = word
            self.value = None

class Command(ob.Object):

    def __init__(self):
        super().__init__()
        self._error = None
        self._func = None
        self._result = []
        self._thrs = []
        self.args = []
        self.batch = False
        self.cc = ""
        self.channel = ""
        self.cfrom = ""
        self.chk = ""
        self.delta = 0
        self.dkeys = []
        self.ignore = ""
        self.index = None
        self.match = None
        self.options = ""
        self.orig = None
        self.origin = ""
        self.rest = ""
        self.selector = {}
        self.setter = {}
        self.txt = ""

    def get_aliased(self, txt):
        spl = txt.split()
        if spl and spl[0] in ob.tables.aliases:
            cmd = spl[0]
            v = ob.tables.aliases.get(cmd, None)
            if v:
                spl[0] = cmd.replace(cmd, v)
        txt2 = " ".join(spl)
        return txt2 or txt

    def get_tokens(self):
        words = self.txt.split()
        tokens = []
        nr = -1
        for word in words:
            nr += 1
            token = Token()
            token.parse(nr, word)
            tokens.append(token)
        return tokens

    def parse(self, txt, options=""):
        if not txt:
            return
        from .utils import parse_date, to_day
        txt = txt.replace("\u0001", "")
        txt = txt.replace("\001", "")
        if txt and self.cc == txt[0]:
            txt = txt[1:]
        self.txt = self.get_aliased(txt)
        if not self.txt:
             self.txt = txt
        nr = -1
        self.cfrom = ob.utils.get_from()
        self.args = []
        self.dkeys = []
        self.options = options or self.options or ""
        prev = ""
        for token in self.get_tokens():
             nr += 1
             if token.chk:
                 self.chk = token.chk
             if token.match:
                 self.match = token.match
             if token.index:
                 self.index = token.index
                 self.args.append(token.arg)
                 continue
             if token.option:
                 self.options += "," + token.option
             if prev == "-o":
                 prev = ""
                 self.options += "," + token.value
             if token.down:
                 prev = token.down
             if token.ignore:
                 self.ignore = token.ignore
             elif token.dkey:
                 self.dkeys.append(token.dkey)
             if token.selector:
                 self.selector[token.selector] = token.value
             if token.setter:
                 self.setter[token.setter] = token.value
             if token.up:
                 self.delta = parse_date(token.up)
             elif token.down:
                 self.delta = parse_date(token.down)
             if token.arg:
                 self.args.append(token.arg)
        for opt in self.options.split(","):
            try:
                self.index = int(opt)
                break
            except ValueError:
                 pass
            continue
        self.rest = " ".join(self.args)
        self.time = to_day(self.rest)
