""" filebased JSON objects. """

__version__ = 1

## imports

import logging
import ob
import os
import time

## defines

def __dir__():
    return ("Cursor", "last")

## classes

class Cursor(ob.Object):

    def all(self, type, selector={}, index=None, delta=0, showdel=False):
        nr = -1
        for fn in filenames(type, delta):
            nr += 1
            o = ob.cached(fn)
            if index is not None and nr != index:
                continue
            if not showdel and ("_deleted" in dir(o) and o._deleted):
                continue
            if selector:
                if ob.search(o, selector):
                    yield o
            else:
                yield o

    def find(self, type, selector, index=None, delta=0, showdel=False):
        nr = -1
        for fn in filenames(type, delta):
            try:
                o = ob.cached(fn)
            except ob.errors.EJSON:
                logging.error("EJSON %s" % fn)
                continue
            if not o:
                continue
            if not showdel and ("_deleted" in dir(o) and o._deleted):
                continue
            if ob.search(o, selector):
                nr += 1
                if index is not None and nr != index:
                    continue
                yield o

    def last(self, type, selector={},index=None, delta=0, showdel=False):
        res = []
        nr = -1
        for fn in filenames(type, delta):
            o = ob.cached(fn)
            if not o:
                continue
            if not showdel and ("_deleted" in dir(o) and o._deleted):
                continue
            if selector and ob.search(o, selector):
                nr += 1
                if index is not None and nr != index:
                    continue
                res.append((fn, o))
            else:
                res.append((fn, o))
        if res:
            s = sorted(res, key=lambda x:fn_time(x[0]))
            if s:
                return s[-1][-1]
        return None

    def last_fn(self, type, index=None, delta=0):
        fns = filenames(type, delta)
        if fns:
            fn = fns[-1]
            return (fn, ob.filehook(fn))
        return (None, None)

## functions

def days(path):
    return elapsed(time.time() - fn_time(path))

def elapsed(seconds, short=True):
    txt = ""
    nsec = float(seconds)
    year = 365*24*60*60
    week = 7*24*60*60
    nday = 24*60*60
    hour = 60*60
    minute = 60
    years = int(nsec/year)
    nsec -= years*year
    weeks = int(nsec/week)
    nsec -= weeks*week
    nrdays = int(nsec/nday)
    nsec -= nrdays*nday
    hours = int(nsec/hour)
    nsec -= hours*hour
    minutes = int(nsec/minute)
    sec = nsec - minutes*minute
    if years:
        txt += "%sy" % years
    if weeks:
        nrdays += weeks * 7
    if nrdays:
        txt += "%sd" % nrdays
    if years and short and txt:
        return txt
    if hours:
        txt += "%sh" % hours
    if nrdays and short and txt:
        return txt
    if minutes:
        txt += "%sm" % minutes
    if hours and short and txt:
        return txt
    if sec == 0:
        txt += "0s"
    elif sec < 1 or not short:
        txt += "%.3fs" % sec
    else:
        txt += "%ss" % int(sec)
    txt = txt.strip()
    return txt

def filenames(name, delta=0):
    if not name:
        return []
    assert ob.workdir
    p = os.path.join(ob.workdir, "store", name) + os.sep
    res = []
    now = time.time()
    past = now + delta
    for rootdir, dirs, files in os.walk(p, topdown=True):
        if name and name not in rootdir:
            continue
        for fn in files:
            fnn = os.path.join(rootdir, fn).split(os.path.join(ob.workdir, "store"))[-1]
            if delta:
                if fn_time(fnn) < past:
                    continue
            res.append(os.sep.join(fnn.split(os.sep)[1:]))
    return sorted(res, key=lambda x: fn_time(x))

def fn_time(daystr):
    daystr = daystr.replace("_", ":")
    datestr = " ".join(daystr.split(os.sep)[-2:])
    try:
        datestr, rest = datestr.rsplit(".", 1)
    except ValueError:
        rest = ""
    try:
        t = time.mktime(time.strptime(datestr, "%Y-%m-%d %H:%M:%S"))
        if rest:
            t += float("." + rest)
    except ValueError:
        t = 0
    return t

def last(o, s={}):
    db = Cursor()
    oo = db.last(ob.get_type(o), s)
    o.update(oo)

def last_fn(type):
    fns = filenames(type)
    if fns:
        return fns[-1]
