""" kernel central registry. """

__version__ = 1

def __dir__():
    return ("homedir", "cmd", "default", "opts", "parse_cli", "daemon")

## imports

import logging
import ob
import optparse
import os
import sys
import time

## defines

homedir = os.path.expanduser("~")

default = ob.Object()
default.background = False
default.debug = False
default.level = "error"
default.modules = ""
default.owner = "root@shell"
default.resume = False
default.shell = False
default.verbose = False
default.workdir = os.path.join(homedir, ".ob")

opts = [
    ('-b', '', 'store_true', False, 'background', 'enable daemon mode.'),
    ('-d', '', 'string', "", 'workdir', 'set working directory.'),
    ('-l', '', 'string', 'error', 'level', 'loglevel.'),
    ('-m', '', 'string', '', 'modules', 'modules to load.'),
    ('-o', '', "string", "", 'options', "options to use."),
    ('-r', '', "store_true", False, 'resume', 'resume from filedescriptor.'),
    ('-v', '', "store_true", False, 'verbose', 'verbose mode.'),
    ('-x', '', "string", "", 'exclude', "modules to exclude from loading."),
    ('-z', '', "store_true", False, 'shell', "enable shell."),
    ('', '--debug', "store_true", False, 'debug', 'enable debug mode.'),
    ('', '--owner', "string", "", 'owner', "owner's userhost or JID."),
    ('', '--password', "string", "", 'password', "password to use"),
]

## functions

def make_opts(options, usage, version):
    parser = optparse.OptionParser(usage=usage, version=version)
    for opt in options:
        if not opt:
            continue
        otype, deft, dest, htype = opt[2:]
        if "store" in otype:
            parser.add_option(opt[0], opt[1], action=otype, default=deft, dest=dest, help=htype)
        else:
            parser.add_option(opt[0], opt[1], type=otype, default=deft, dest=dest, help=htype)
    return parser.parse_args()

def parse_cli(name="ob", version=__version__, wd="", usage=""):
    from ob import k
    ver = "%s %s" % (name.upper(), version)
    usage = "%s -m [mod1,mod2] cmd"  % name
    try:
        opt, arguments = make_opts(sorted(opts), usage, ver)
    except SystemExit:
        opt = ob.Cfg(default)
        arguments = []
    cfg = ob.Cfg()
    cfg.update(vars(opt))
    cfg.name = name
    cfg.version = version
    cfg.args = arguments
    cfg.workdir = wd or cfg.workdir or os.path.join(homedir, ".ob")
    sp = os.path.join(cfg.workdir, "store") + os.sep
    if not os.path.exists(sp):
        ob.utils.cdir(sp)
    ob.workdir = cfg.workdir
    ob.k.cfg.update(cfg)
    ob.log.level(cfg.level or "error")
    logging.warning("%s started (%s) at %s" % (cfg.name.upper(), cfg.level, time.ctime(time.time())))
    return cfg
 
def cmd(txt, origin=""):
    from .bot import Bot
    from .event import Event
    from ob import k
    b = Bot()
    b.cfg.prompt = False
    b.start()
    b.initialize(k.cfg.modules)
    e = Event()
    e.txt = txt
    e.options = k.cfg.options
    e.orig = repr(b)
    e.origin = origin or "root@shell"
    b.dispatch(e)
    e.wait()
    return e

def daemon():
    pid = os.fork()
    if pid != 0:
        reset()
        os._exit(0)
    os.setsid()
    os.umask(0)
    si = open("/dev/null", 'r')
    so = open("/dev/null", 'a+')
    se = open("/dev/null", 'a+')
    os.dup2(si.fileno(), sys.stdin.fileno())
    os.dup2(so.fileno(), sys.stdout.fileno())
    os.dup2(se.fileno(), sys.stderr.fileno())

def writepid():
    from ob import k
    path = os.path.join(k.cfg.workdir, "pidfile")
    f = open(path, 'w')
    f.write(str(os.getpid()))
    f.flush()
    f.close()
