""" threads. """

__version__ = 1

## imports

import ob
import logging
import queue
import threading
import time
import types

## defines

def __dir__():
    return ("Launcher", "Task", "get_name", "launch", "kill")

## classes

class Task(threading.Thread):

    def __init__(self, func, *args, name="noname", daemon=True):
        super().__init__(None, self.run, name, (), {}, daemon=daemon)
        self._result = None
        self._queue = queue.Queue()
        self._queue.put((func, args))

    def __iter__(self):
        return self

    def __next__(self):
        for k in dir(self):
            yield k

    def run(self):
        func, args = self._queue.get()
        try:
            self._result = func(*args)
            if args:
                args[0].ready()
        except Exception as ex:
            logging.error(ob.utils.get_exception())

    def join(self):
        super().join()
        return self._result

class Launcher(object):

    def launch(self, func, *args):
        name = ""
        try:
            name = args[0].txt
        except (AttributeError, IndexError):
            name = get_name(func)
        t = Task(func, *args, name=name)
        t.start()
        return t

## functions

def get_name(o):
    t = type(o)
    if t == types.ModuleType:
        return o.__name__
    try:
        n = "%s.%s" % (o.__self__.__class__.__name__, o.__name__)
    except AttributeError:
        try:
            n = "%s.%s" % (o.__class__.__name__, o.__name__)
        except AttributeError:
            try:
                n = o.__class__.__name__
            except AttributeError:
                n = o.__name__
    return n
