""" terminal code. """

__version__ = 1

## imports

import atexit
import os
import readline
import rlcompleter
import sys
import termios

## defines

def __dir__():
    return ("close_history", "reset", "set_completer", "termsave") 

ERASE_LINE = '\033[2K'
BOLD='\033[1m'
RED = '\033[91m'   
YELLOW = '\033[93m'
GREEN = '\033[92m' 
BLUE = '\033[94m'
BLA = '\033[95m'
ENDC = '\033[0m'

## runtime

resume = {}
cmds = []
args = []

## functions

def close_history():
    from ob import k
    try:
        histfile = os.path.abspath(os.path.join(k.cfg.workdir, "history"))
        readline.write_history_file(histfile)
    except:
        pass

def complete(text, state):
    matches = []
    if text:
         matches = [s for s in cmds if s and s.startswith(text)]
    else:
         matches = cmds[:]
    try:
        return matches[state]
    except IndexError:
        return None

def enable_history():
    histfile = os.path.abspath(os.path.join(k.cfg.workdir, "history"))
    if os.path.exists(histfile):
        readline.read_history_file(histfile)

def get_completer():
    return readline.get_completer()

def reset():
    close_history()
    if "old" in resume:
        termreset(resume["fd"], resume["old"])

def set_completer(commands, arguments):
    global cmds
    global args
    cmds = commands
    args = arguments
    readline.set_completer(complete)
    readline.parse_and_bind("tab: complete")
    atexit.register(lambda: readline.set_completer(None))

def termsave():
    resume["fd"] = sys.stdin.fileno()
    resume["old"] = termsetup(sys.stdin.fileno())
    atexit.register(reset)

def termreset(fd, old):
    termios.tcsetattr(fd, termios.TCSADRAIN, old)

def termsetup(fd):
    old = termios.tcgetattr(fd)
    return old
