""" users management. """

__version__ = 1

## import

import ob
import ob.db
import logging
import time

## defines

def __dir__():
    return ("User", "Users")

## classes

class User(ob.Object):

    def __init__(self):
        super().__init__()
        self.user = ""
        self.perms = []

class Users(ob.Object):

    cache = ob.Object()
    db = ob.db.Cursor()
    userhosts = ob.Object()

    def allowed(self, origin, perm):
        perm = perm.upper()
        user = self.get_user(origin)
        if user:
            if perm in user.perms:
                return True
        logging.error("denied %s %s" % (origin, perm))
        return False

    def delete(self, origin, perm):
        for user in self.get_users(origin):
            try:
                user.perms.remove(perm)
                user.save()
                return True
            except ValueError:
                pass

    def get_users(self, origin=""):
        s = {"user": origin}
        for o in self.all("ob.user.User", s):
            return o

    def get_user(self, origin, type=None):
        u = Users.cache.get(origin, None)
        if u:
            return u
        s = {"user": origin}
        for o in self.db.find("ob.user.User", s):
            Users.cache.set(origin, o)
            return o

    def meet(self, origin, perms=[]):
        user = self.get_user(origin)
        if not user:
            user = User()
        user.user = origin
        user.perms = perms + ["USER", ]
        if perms:
            user.perms.extend(perms)
        user.save(timed=True)
        return user

    def oper(self, origin):
        user = User()
        user.user = origin
        user.perms = ["OPER", "USER"]
        setattr(Users.cache, origin, user)
        return user

    def perm(self, origin, permission):
        user = self.get_user(origin)
        if not user:
            raise ENOUSER(origin)
        if permission.upper() not in user.perms:
            user.perms.append(permission.upper())
            user.save()
        return user
